/***************************************************************************
                         helpdialog.cpp  -  description
                             -------------------
    begin                : Fr Nov 15 13:44:19 CEST 2001
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "helpdialog.h"

// Qt includes
#include <qcombobox.h>
#include <qheader.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

// KDE includes
#include <klocale.h>
#include <klistview.h>
#include <kpushbutton.h>

HelpDialog::HelpDialog( QLineEdit* lineedit, QWidget* parent,
    const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl ), text( lineedit )
{
    resize( 318, 436 ); 
    setCaption( i18n( "Help" ) );
    HelpDialogLayout = new QVBoxLayout( this, 11, 6, "HelpDialogLayout"); 

    comboHeadline = new QComboBox( FALSE, this, "comboHeadline" );
    HelpDialogLayout->addWidget( comboHeadline );

    list = new KListView( this, "list" );
    list->addColumn( i18n( "Token" ) );
    list->addColumn( i18n( "Description" ) );
    HelpDialogLayout->addWidget( list );

    Layout1 = new QHBoxLayout( 0, 0, 6, "Layout1"); 
    QSpacerItem* spacer = new QSpacerItem( 91, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout1->addItem( spacer );

    buttonAdd = new KPushButton( this, "buttonAdd" );
    buttonAdd->setText( i18n( "&Add" ) );
    Layout1->addWidget( buttonAdd );

    buttonClose = new KPushButton( this, "buttonClose" );
    buttonClose->setText( i18n( "&Close" ) );
    Layout1->addWidget( buttonClose );
    HelpDialogLayout->addLayout( Layout1 );

    // signals and slots connections
    connect( buttonClose, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( exec()  ) );
    connect( list, SIGNAL( executed(QListViewItem*) ), this, SLOT( exec() ) );
    connect( comboHeadline, SIGNAL( activated(int) ), this, SLOT(updateItems() ) );
}

HelpDialog::~HelpDialog()
{
}

void HelpDialog::add( QString _command, QString _help )
{
    head.append( curheadline );
    command.append( _command );
    help.append( _help );

    updateItems();
}

void HelpDialog::remove( QString _command )
{
    for( unsigned int i = 0; i < command.count(); i++ )
        if( command[i] == _command ) {
            command.remove( command.at( i ) );
            head.remove( head.at( i ) );
            help.remove( help.at( i ) );                        
        }
    
    updateItems();
}

void HelpDialog::addHeadline( QString headline )
{
    curheadline = headline;
    comboHeadline->insertItem( headline );
    updateItems();    
}

void HelpDialog::removeHeadline( QString headline )
{
    for( int i = 0; i < comboHeadline->count(); i++ )
        if( comboHeadline->text(i) == headline ) {
            comboHeadline->removeItem( i );
            break;
        }
    updateItems();        
}

void HelpDialog::exec()
{
    if(!list->currentItem())
        return;

    QString t = text->text();

    t.insert( text->cursorPosition(), list->currentItem()->text( 0 ) );
    text->setText( t );
}

void HelpDialog::updateItems()
{
    list->clear();
    for( unsigned int i = 0; i < head.count(); i++ )
        if( head[i] == comboHeadline->currentText() ) {
            new KListViewItem( list, command[i], help[i] );
        }
}
