/***************************************************************************
                          pictureplugin.cpp  -  description
                             -------------------
    begin                : Son Apr 14 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pictureplugin.h"

// QT includes
#include <qfile.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qstring.h>
#include <qvariant.h>
#include <qwidget.h>

// KDE includes
#include <klocale.h>

QString PicturePlugin::getName()
{
    return i18n("Picture Plugin");
}

QString PicturePlugin::getAccelName()
{
    return i18n("P&icture Plugin");
}

int PicturePlugin::type()
{
    return TYPE_BRACKET;
}

bool PicturePlugin::checkError()
{
    return true;
}

void PicturePlugin::drawInterface( QWidget* w, QVBoxLayout* l )
{
    QHBoxLayout* hb = new QHBoxLayout( 0, 0, 6 );
    QLabel* la = new QLabel( w );
    la->setText( i18n("<b>Picture Plugin</b>") );
    l->addWidget( la );

    la = new QLabel( w );
    la->setText( i18n( "Rename pictures using information like their resolution or color mode.") );
    l->addWidget( la );

    la = new QLabel( w );
    la->setText( i18n("This plugin supports the following tokens:") );
    l->addWidget( la );

    la = new QLabel( w );
    la->setText(i18n("<b>[resolution]</b> insert resolution<br>"
                    "<b>[xres]</b> insert x resolution<br>" ) );
    hb->addWidget( la );

    la = new QLabel( w );
    la->setText(i18n("<b>[yres]</b> insert y resolution<br>"
                    "<b>[bitdepth]</b> insert bitdepth<br>" ) );
    hb->addWidget( la );
    l->addLayout( hb );

    QStringList list = QImage::inputFormatList();
    QString tmp = "";
    for( unsigned int i = 0; i < list.count(); i++ ) {
        if( i < list.count() && i )
            tmp += ", ";

        if( i % 8 == 0 && i < list.count() )
            tmp += "<br>";
            
        tmp += list[i];
    }

    la = new QLabel( w );
    la->setText( i18n( "Supported image formats: " ) + tmp );
    l->addWidget( la );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    l->addItem( spacer );
}

void PicturePlugin::fillStructure()
{
    /* Don't care about this */
}

QString PicturePlugin::processFile( values* val, int i, QString token, int mode )
{
    resolution = xres = yres = bitdepth =  "";
    QString filename = val->files[i].source_dir + val->files[i].source_name + val->files[i].extension;
    QString extension = filename.right(4).lower();

    token = token.lower();

    // Bad code follows:
    if( token != "resolution" && token != "xres" && token != "yres" && 
        token != "bitdepth" )
        return "";

    QImage* img = new QImage( filename );
    if( !img ) {
        delete img;
        return "";
    }

    resolution = QString( "%1x%2" ).arg(img->width()).arg(img->height());
    xres = QString( "%1" ).arg( img->width() );
    yres = QString( "%1" ).arg( img->height() );
    bitdepth = QString( "%1" ).arg( img->depth() );

    delete img;

    if( token == "resolution" )
        return resolution;
    else if( token == "xres" )
        return xres;
    else if( token == "yres" )
        return yres;
    else if( token == "bitdepth" )
        return bitdepth;

    return "";
}

void PicturePlugin::finished()
{
    // We don't care about this event!
    return;
}

bool PicturePlugin::defaultVisible()
{
    return false;
}

void PicturePlugin::addHelp( HelpDialog* combo )
{
    combo->addHeadline( getName() );
    combo->add( "[resolution]", i18n( "insert resolution" ) );
    combo->add( "[xres]", i18n( "insert x resolution" ) );
    combo->add( "[yres]", i18n( "insert y resolution" ) );
    combo->add( "[bitdepth]", i18n( "insert bitdepth" ) );
}

void PicturePlugin::removeHelp( HelpDialog* combo )
{
    combo->removeHeadline( getName() );
    combo->remove( "[resolution]" );
    combo->remove( "[xres]" );
    combo->remove( "[yres]" );
    combo->remove( "[bitdepth]" );
}
