/***************************************************************************
                          skipdialog.cpp  -  description
                             -------------------
    begin                : Die Mai 15 15:34:19 CEST 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "skipdialog.h"

// QT includes
#include <qlabel.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qtooltip.h>

// KDE includes
#include <kapplication.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kpushbutton.h>

KMyIntSpinBox::KMyIntSpinBox( QWidget* parent )
    : KIntSpinBox( parent )
{ }

KMyIntSpinBox::~KMyIntSpinBox()
{ }

void KMyIntSpinBox::keyPressEvent( QKeyEvent* e )
{
    if( e->key() == Key_Return )
        emit returnPressed();
}

// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

SkipDialog::SkipDialog( QValueList<int> n, int start, QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    resize( 260, 200 );
    setCaption( i18n( "Skip Numbers" ) );

    SkipDialogLayout = new QHBoxLayout( this, 11, 6 );
    Layout1 = new QVBoxLayout( 0, 0, 6 );
    Layout2 = new QHBoxLayout( 0, 0, 6 );
    Layout4 = new QVBoxLayout( 0, 0, 6 );
    Layout5 = new QHBoxLayout( 0, 0, 6 );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );

    listNumbers = new QListBox( this );
    QToolTip::add( listNumbers, i18n("Add all numbers that should be skipped by krename during the rename process.<br>"
    "E.g.: If 2 is skipped files will be numbered: file0, file1, file3, ...") );

    buttonAdd = new KPushButton( this );
    buttonAdd->setText( i18n( "&Add Number" ) );

    buttonRemove = new KPushButton( this );
    buttonRemove->setText( i18n( "&Remove Number" ) );

    buttonSave = new KPushButton( this );
    buttonSave->setText( i18n( "&Save Settings" ) );

    spinNumber = new KMyIntSpinBox( this );
    spinNumber->setMaxValue( SPINMAX+1 );
    spinNumber->setMinValue( start );
    spinNumber->setValue( start );
    spinNumber->setFocus();

    buttonOk = new KPushButton( this );
    buttonOk->setText( i18n( "&OK" ) );
    buttonOk->setIconSet( SmallIconSet("ok") );
    
    buttonCancel = new KPushButton( this );
    buttonCancel->setText( i18n( "&Cancel" ) );

    Layout4->addWidget( buttonRemove );
    Layout4->addWidget( buttonAdd );
    Layout4->addWidget( spinNumber );
    Layout4->addItem( spacer );
    Layout2->addWidget( listNumbers );
    Layout2->addLayout( Layout4 );
    Layout5->addWidget( buttonSave );
    Layout5->addItem( spacer_2 );
    Layout5->addWidget( buttonOk );
    Layout5->addWidget( buttonCancel );
    Layout1->addLayout( Layout2 );
    Layout1->addLayout( Layout5 );
    SkipDialogLayout->addLayout( Layout1 );

    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( addNumber() ) );
    connect( buttonRemove,SIGNAL( clicked() ), this, SLOT( removeNumber() ) );
    connect( spinNumber, SIGNAL( returnPressed()), this, SLOT( addNumber() ) );
    connect( buttonSave, SIGNAL( clicked()), this, SLOT( save() ) );

    for( unsigned int i = 0; i < n.count(); i++ )
            listNumbers->insertItem( QString("%1").arg(n[i]), -1 );
}

SkipDialog::~SkipDialog()
{
}

void SkipDialog::addNumber()
{
    QString tmp = QString("%1").arg(spinNumber->value());
    for( unsigned int i = 0; i < listNumbers->count(); i++ )
        if( listNumbers->text(i) == tmp )
            return;
    listNumbers->insertItem( tmp, -1 );
    sort();
}

void SkipDialog::removeNumber()
{
    unsigned int i = 0;
    do {
        if(listNumbers->isSelected( i )) {
            listNumbers->removeItem( i );
            return;
        } else
            i++;
    } while( i < listNumbers->count() );
}

QValueList<int> SkipDialog::getList()
{
    QValueList<int> skip;
    for( unsigned int i = 0; i < listNumbers->count(); i++ )
        skip.append( listNumbers->text(i).toInt() );

    return skip;
}

void SkipDialog::sort()
{
    // Not very fast, but I hope it won't be used on ot big lists ;)
    if( listNumbers->count() < 1 )
        return;

    for( unsigned int i = 0; i < listNumbers->count()-1; i++) {
        if( listNumbers->text(i).toInt() > listNumbers->text(i+1).toInt() ) {
            QString tmp = listNumbers->text(i);
            listNumbers->removeItem(i);
            listNumbers->insertItem( tmp, i+1 );
            i = 0;
        }
    }
}

void SkipDialog::save()
{
    KConfig* conf = kapp->config();
    QValueList<int> skip = getList();
    conf->setGroup("GUISettings");
    conf->writeEntry( "skip", skip );
    conf->sync();
}

