/***************************************************************************
                          wizard.cpp  -  description
                             -------------------
    begin                : Die Mai 15 15:34:19 CEST 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Own includes
#include "ProgressDialog.h"
#include "batchrenamer.h"
#include "confdialog.h"
#include "fileoperation.h"
#include "krename.h"
#include "kmylistbox.h"
#include "kmylistview.h"
#include "kmyhistorycombo.h"
#include "mylabel.h"
#include "pluginloader.h"
#include "skipdialog.h"
#include "replacedialog.h"
#include "undodialog.h"
#include "helpdialog.h"
#include "coorddialog.h"

// KDE includes
#include <kaboutapplication.h>
#include <kapplication.h>
#include <kaction.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <khelpmenu.h>
#include <kiconloader.h>
#include <klocale.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>
#include <kmenubar.h>
#include <kpushbutton.h>
#include <kprocess.h>
#include <kpopupmenu.h>
#include <kstandarddirs.h>
#include <kstartupinfo.h>
#include <kurl.h>
#include <kurlrequester.h>
#include <kwizard.h>

// QT includes
#include <qabstractlayout.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qradiobutton.h>
#include <qscrollview.h>
#include <qtabwidget.h>
#include <qvaluelist.h>
#include <qwidget.h>
#include <qtooltip.h>
#include <qvalidator.h>

// OS includes
#include <pwd.h>
#include <sys/types.h>
#include <grp.h>
#include <unistd.h>

// Own includes
#include "wizard.h"

using std::vector;

const QString pageTitle[] = {
    i18n( "Files" ),  //i18n( "Adding Files" );
    i18n( "Destination" ),
    i18n( "Plugins" ), //i18n( "Advanced Options" );
    i18n( "Filename" )
};

#define ID_PAGE_1 115
#define ID_PAGE_2 116
#define ID_PAGE_3 117

#define PAGES 4

wizard::wizard( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : KWizard( parent, name, modal, fl )
{
    setCaption( "Krename" ); // "Krename by Dominik Seichter
    setIcon( BarIcon( "krename" ) );

    // Load Plugins
    plugin = new PluginLoader();
    menuBar = new KMenuBar(this);

    setupActions();

    // to render the correct titlebar
    KConfig* config = kapp->config();
    config->setGroup("WizardPages");
    m_title = config->readBoolEntry("AdvancedTitle", true );

    setupPages();
    
    connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( addFile() ));
    connect( buttonAddDir, SIGNAL( clicked() ), this, SLOT( addDir() ));
    connect( buttonAddDirRec, SIGNAL( clicked() ), this, SLOT( addDirRec() ));
    connect( buttonAddDirName, SIGNAL( clicked() ), this, SLOT( addDirName() ));
    connect( buttonRemoveAll, SIGNAL( clicked()), this, SLOT( clearList() ));
    connect( buttonRemove, SIGNAL( clicked()), this, SLOT( removeFile() ));
    connect( buttonAbout, SIGNAL( clicked()), this, SLOT( about() ));
    connect( buttonHelp, SIGNAL( clicked()), this, SLOT( showTokenHelp() ));
    connect( this->finishButton(), SIGNAL( clicked()), this, SLOT( start() ));

    connect( checkExtension, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionRename, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionMove, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionCopy, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionLink, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( checkUndoScript, SIGNAL( clicked()), this, SLOT( enableControls() ));

    connect( fileList, SIGNAL( updateCount()), this, SLOT( updateCount() ));
    connect( fileList, SIGNAL( updatePreview()), this, SLOT( updatePreview() ));
    connect( buttonUp, SIGNAL( clicked()), fileList, SLOT( moveUp() ));
    connect( buttonDown, SIGNAL( clicked()), fileList, SLOT( moveDown() ));
    connect( buttonUp, SIGNAL( clicked()), this, SLOT( updatePreview() ));
    connect( buttonDown, SIGNAL( clicked()), this, SLOT( updatePreview() ));
    connect( buttonUp2, SIGNAL( clicked()), this, SLOT( moveUp() ));
    connect( buttonDown2, SIGNAL( clicked()), this, SLOT( moveDown() ));
    connect( buttonSkip, SIGNAL( clicked()), this, SLOT( skipDialog() ));
    connect( buttonReplace, SIGNAL( clicked()), this, SLOT( replace() ));
    connect( buttonCoord, SIGNAL( clicked()), this, SLOT( getCoordinates() ));

    connect( filename, SIGNAL( textChanged(const QString &)), this, SLOT( updatePreview() ));
    connect( extemplate, SIGNAL( textChanged(const QString &)), this, SLOT( updatePreview() ));
    connect( spinStep, SIGNAL( valueChanged(int)), this, SLOT( updatePreview() ));
    connect( spinIndex, SIGNAL( valueChanged(int)), this, SLOT( updatePreview() ));
    connect( checkExtension, SIGNAL( clicked()), this, SLOT( updatePreview() ));
    connect( checkPreview, SIGNAL( clicked()), this, SLOT( toggleImagePreview() ));
    connect( buttonConf, SIGNAL(clicked()), this, SLOT( preferences() ));
    connect( preview, SIGNAL( itemChanged() ), this, SLOT( updatePreview() ));
    connect( preview, SIGNAL( removeItem(int) ), this, SLOT( removeFile(int) ));
    connect( preview, SIGNAL( addFile() ), this, SLOT( addFile() ));
    connect( preview, SIGNAL( addDir() ), this, SLOT( addDir() ));
    connect( undorequester, SIGNAL( urlSelected(const QString &)), this, SLOT( changeUndoScriptName() ));
    connect( checkShowAll, SIGNAL( clicked() ), this, SLOT( updatePluginTab() ) );
    connect( kapp, SIGNAL( aboutToQuit() ), this, SLOT( saveConfig() ) );
    connect( buttonMove, SIGNAL( clicked()), fileList, SLOT( moveMode() ));
    connect( checkName, SIGNAL( clicked()), this, SLOT( toggleName() ));

    show();
    
    // Load Configuration
    loadConfig();

    filename->setText( "$" );   // no i18n() ! ;)

    /*
     * Special case:
     * start up and do not display page 1 !
     * BTW, I HATE THIS CODE!
     */

    if( !mnuView->isItemChecked( ID_PAGE_1 ) ) {
        if( mnuView->isItemChecked( ID_PAGE_2 ) )
            showPage( page_2 );
        else if( mnuView->isItemChecked( ID_PAGE_3 ) )
            showPage( page_3 );
        else
            showPage( page_4 );
    }

    // Tell KStartupInfo that KRename has been loaded completly
    KStartupInfoId id;
    id.initId( kapp->startupId() );
    KStartupInfo::sendFinish( id );
}

wizard::~wizard()
{
}

bool wizard::noRealTimePreview = false;
QString wizard::lastPath = QString::null;

void wizard::setupActions()
{
    KActionCollection* actionCollection = new KActionCollection( this );

    KPopupMenu* mnuFile = new KPopupMenu( this );
    KPopupMenu* mnuExtra = new KPopupMenu( this );
    KPopupMenu* mnuSettings = new KPopupMenu( this );
    KHelpMenu* mnuHelp = new KHelpMenu( this );
    mnuView = new KPopupMenu( this );
    
    menuBar->insertItem( i18n("&File"), mnuFile );
    menuBar->insertItem( i18n("E&xtras"), mnuExtra );
    mnuExtra->insertItem( SmallIcon("undo"), i18n("&Undo Old Renaming Action..."), this, SLOT( undo() ) );
    menuBar->insertItem( i18n("&Settings"), mnuSettings );
    menuBar->insertSeparator();
    menuBar->insertItem( i18n("&Help"), mnuHelp->menu() );

    mnuSettings->insertItem( i18n("&Visible Pages"), mnuView );
    mnuSettings->insertSeparator();
    
    mnuView->insertItem( pageTitle[0], this, SLOT( pageVisible1() ), 0, ID_PAGE_1 );
    mnuView->insertItem( pageTitle[1], this, SLOT( pageVisible2() ), 0, ID_PAGE_2 );
    mnuView->insertItem( pageTitle[2], this, SLOT( pageVisible3() ), 0, ID_PAGE_3 );

    KAction* openAct = KStdAction::open(this, SLOT(addFile()), actionCollection);
    KAction* quitAct = KStdAction::quit(this, SLOT(reject()), actionCollection);
    KAction* prefAct = KStdAction::preferences( this, SLOT(preferences()), actionCollection );
    
//    KAction* spellAct = KStdAction::spelling( this, SLOT(spellCheck()), actionCollection, "spell" );
//    spellAct->plug( mnuExtra );
    
    openAct->plug( mnuFile );
    mnuFile->insertSeparator();
    quitAct->plug( mnuFile );

    prefAct->plug( mnuSettings );
    
    connect( mnuHelp, SIGNAL(showAboutApplication()), this, SLOT(about()));
    connect( mnuView, SIGNAL(activated(int)), this, SLOT(visiblePage(int)));
}

void wizard::setupPages()
{
    if( m_title ) {
        topLabel1.resize( PAGES );
        topLabel2.resize( PAGES );
        topLabel3.resize( PAGES );
        topLabel4.resize( PAGES );
    }

    setupPage1();
    setupPage2();
    setupPage3();
    setupPage4();

    // Disable dirname ;)
    enableControls();

    setFinish( page_4, TRUE );
}

void wizard::setupPage1()
{
    page_1 = new QWidget( this );

    pageLayout = new QVBoxLayout( page_1, 11, 6 );
    Layout1 = new QVBoxLayout( 0, 0, 6 );
    Layout2 = new QHBoxLayout( 0, 0, 6 );
    Layout3 = new QVBoxLayout( 0, 0, 6 );
    Layout4 = new QHBoxLayout( 0, 0, 6 );
    Layout5 = new QVBoxLayout( 0, 0, 6 );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    QSpacerItem* spacer2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );

    addTitle( page_1, pageLayout, pageTitle[0] );
    fileList = new KMyListBox( page_1 );

    buttonAdd = new KPushButton( page_1 );
    buttonAdd->setText( i18n( "&Add Files" ) );

    buttonAddDir = new KPushButton( page_1 );
    buttonAddDir->setText( i18n( "Add &Directory" ) );

    buttonAddDirRec = new KPushButton( page_1 );
    buttonAddDirRec->setText( i18n( "Add Dir Recursivel&y" ) );

    buttonAddDirName = new KPushButton( page_1 );
    buttonAddDirName->setText( i18n("&Rename Directory") );

    buttonRemove = new KPushButton( page_1 );
    buttonRemove->setText( i18n( "&Remove Files" ) );

    buttonRemoveAll = new KPushButton( page_1 );
    buttonRemoveAll->setText( i18n( "R&emove All" ) );

    checkPreview = new QCheckBox( page_1 );
    checkPreview->setText( i18n("&Preview") );
    
    labelCount = new QLabel( page_1 );
    updateCount();

    buttonUp = new KPushButton( page_1 );
    buttonUp->setPixmap( BarIcon( "1uparrow" ) );

    buttonDown = new KPushButton( page_1 );
    buttonDown->setPixmap( BarIcon( "1downarrow" ) );

    checkName = new QCheckBox( page_1 );
    checkName->setText( i18n("&Display name") );

    buttonMove = new KPushButton( page_1 );
    buttonMove->setPixmap( BarIcon( "move" ) );

    buttonAbout = new KPushButton( page_1 );
    buttonAbout->setText( i18n( "A&bout" ) );

    Layout2->addWidget( fileList );
    Layout2->setStretchFactor( fileList, 2 );

    Layout3->addWidget( buttonAdd );
    Layout3->addWidget( buttonAddDir );
    Layout3->addWidget( buttonAddDirRec );
    Layout3->addWidget( buttonAddDirName );
    Layout3->addWidget( buttonRemove );
    Layout3->addWidget( buttonRemoveAll );
    Layout3->addWidget( checkPreview );
    Layout3->addWidget( checkName );

    Layout3->addWidget( labelCount );
    Layout3->addItem( spacer );

    Layout5->addWidget( buttonUp );
    Layout5->addWidget( buttonMove );
    Layout5->addWidget( buttonDown );
    Layout5->addItem( spacer );

    Layout4->addLayout( Layout5 );
    Layout4->addItem( spacer2 );

    Layout3->addLayout( Layout4 );
    Layout3->addWidget( buttonAbout );
    Layout2->addLayout( Layout3 );
    Layout1->addLayout( Layout2 );

    pageLayout->addLayout( Layout1 );
    addPage( page_1, pageTitle[0] );

    QToolTip::add( buttonAdd, i18n( "Add one or more files" ) );
    QToolTip::add( buttonAddDir, i18n( "Add all files of a directory" ) );
    QToolTip::add( buttonAddDirRec, i18n( "Add all files of a directory recursively,<br>i.e. with all files from all subdirectories" ) );
    QToolTip::add( buttonRemove, i18n( "Remove selected files" ) );
    QToolTip::add( buttonRemoveAll, i18n( "Remove all files" ) );
    QToolTip::add( fileList, i18n( "Drag files or directories into this list to add them.<br>Double click on a file to open it." ) );
    QToolTip::add( checkPreview, i18n( "Enable/disable preview of pictures." ) );
    QToolTip::add( labelCount, i18n( "Displays the number of files in the list." ) );
    QToolTip::add( buttonUp, i18n( "Move selected items up" ));
    QToolTip::add( buttonDown, i18n( "Move selected items down" ));
    QToolTip::add( checkName, i18n( "Enable/disable display of file name." ) );
    QToolTip::add( buttonUp, i18n( "Move selected items (select the new location with the mouse)." ));
}

void wizard::setupPage2()
{
    page_2 = new QWidget( this );
    
    pageLayout_2 = new QVBoxLayout( page_2, 11, 6 );
    Layout7 = new QHBoxLayout( 0, 0, 6 );
    Layout8 = new QVBoxLayout( 0, 0, 6 );

    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );

    addTitle( page_2, pageLayout_2, pageTitle[1] );
    
    groupOptions = new QButtonGroup( page_2 );
    groupOptions->setTitle( i18n( "Options" ) );
    groupOptions->setColumnLayout(0, Qt::Vertical );
    groupOptions->layout()->setSpacing( 6 );
    groupOptions->layout()->setMargin( 11 );
    groupOptionsLayout = new QHBoxLayout( groupOptions->layout() );
    groupOptionsLayout->setAlignment( Qt::AlignTop );

    optionRename = new QRadioButton( groupOptions );
    optionRename->setText( i18n( "&Rename input files" ) );
    optionRename->setChecked( true );

    optionCopy = new QRadioButton( groupOptions );
    optionCopy->setText( i18n( "&Copy files to destination directory" ) );

    optionMove = new QRadioButton( groupOptions );
    optionMove->setText( i18n( "&Move files to destination directory" ) );

    optionLink = new QRadioButton( groupOptions );
    optionLink->setText( i18n( "&Create symbolic links in destination directory" ) );

    checkOverwrite = new QCheckBox( groupOptions );
    checkOverwrite->setText( i18n( "&Overwrite existing files" ) );

    groupDir = new QGroupBox( page_2 );
    groupDir->setTitle( i18n( "Destination Directory" ) );
    groupDir->setColumnLayout(0, Qt::Vertical );
    groupDir->layout()->setSpacing( 6 );
    groupDir->layout()->setMargin( 11 );
    groupDirLayout = new QHBoxLayout( groupDir->layout() );
    groupDirLayout->setAlignment( Qt::AlignTop );

    dirname = new KMyHistoryCombo( groupDir );
    urlrequester = new KURLRequester( dirname, groupDir );
    urlrequester->setMode( KFile::Directory | KFile::ExistingOnly );

    Layout8->addWidget( optionRename );
    Layout8->addWidget( optionCopy );
    Layout8->addWidget( optionMove );
    Layout8->addWidget( optionLink );
    Layout8->addItem( spacer_3 );
    Layout8->addWidget( checkOverwrite );

    groupDirLayout->addWidget( urlrequester );
    groupOptionsLayout->addLayout( Layout8 );

    groupUndo = new QGroupBox( page_2 );
    groupUndo->setTitle( i18n("Undo Renaming") );
    groupUndo->setColumnLayout(0, Qt::Vertical );
    groupUndo->layout()->setSpacing( 6 );
    groupUndo->layout()->setMargin( 11 );
    groupUndoLayout = new QVBoxLayout( groupUndo->layout() );
    groupUndoLayout->setAlignment( Qt::AlignTop );

    checkUndoScript = new QCheckBox( i18n("&Create an undo script"), groupUndo );

    undorequester = new KURLRequester( groupUndo );
    undorequester->setEnabled( false );
    undorequester->setMode( KFile::File | KFile::LocalOnly  );
    undorequester->setFilter( i18n("*.krename|KRename undo scripts (*.krename)\n"
                                   "*|All Files (*)") );
    undorequester->fileDialog()->setOperationMode( KFileDialog::Saving );

    groupUndoLayout->addWidget( checkUndoScript );
    groupUndoLayout->addWidget( undorequester );

    Layout7->addWidget( groupOptions );
    Layout7->addItem( spacer_4 );

    pageLayout_2->addLayout( Layout7 );
    pageLayout_2->addWidget( groupDir );
    pageLayout_2->addWidget( groupUndo );
    pageLayout_2->addItem( spacer_3 );

    addPage( page_2, pageTitle[1] );

    QToolTip::add( optionRename, i18n( "Input files will be renamed." ) );
    QToolTip::add( optionCopy, i18n( "Copies all files to the destination directory and renames them." ) );
    QToolTip::add( optionMove, i18n( "Moves all files to the destination directory and renames them." ) );
    QToolTip::add( optionLink, i18n( "A symbolic link will be created in the destination directory for each file." ) );
}

void wizard::setupPage3()
{
    page_3 = new QWidget( this, "page_3" );
    pageLayout_3 = new QVBoxLayout( page_3, 11, 6 );
    QHBoxLayout* Layout = new QHBoxLayout( 0, 6, 6 );

    addTitle( page_3, pageLayout_3, pageTitle[2] );
   
    tab = new QTabWidget( page_3 );
    
    buttonConf = new KPushButton( page_3 );
    buttonConf->setText( i18n( "&Visible Plugins" ));

    checkShowAll = new QCheckBox( page_3 );
    checkShowAll->setText( i18n("&Show All Plugins" ));

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Layout->addWidget( buttonConf );
    Layout->addWidget( checkShowAll );
    Layout->addItem( spacer );

    setupTab1();

    pageLayout_3->addWidget( tab );
    pageLayout_3->setStretchFactor( tab, 2 );
    pageLayout_3->addLayout( Layout );

    addPage( page_3, pageTitle[2] );
}

void wizard::setupPage4()
{
    page_4 = new QWidget( this );
    pageLayout_4 = new QVBoxLayout( page_4, 11, 6 );

    Layout10 = new QVBoxLayout( 0, 0, 6 );
    Layout11 = new QHBoxLayout( 0, 0, 6 );
    Layout12 = new QVBoxLayout( 0, 0, 6 );
    Layout13 = new QHBoxLayout( 0, 0, 6 );
    Layout17 = new QVBoxLayout( 0, 0, 6 );
    Layout18 = new QHBoxLayout( 0, 0, 6 );
    Layout21 = new QVBoxLayout( 0, 0, 6 );
    Layout22 = new QHBoxLayout( 0, 0, 6 );
    Layout23 = new QVBoxLayout( 0, 0, 6 );

    QSpacerItem* spacer_9 = new QSpacerItem( 200, 20, QSizePolicy::Maximum, QSizePolicy::Maximum );

    addTitle( page_4, pageLayout_4, pageTitle[3] );
    
    labelTemplate = new QLabel( page_4 );
    labelTemplate->setText( i18n( "Template:" ) );

    filename = new KMyHistoryCombo( page_4 );
    filename->setFocus();

    QLabel* labelDot = new QLabel( page_4 );
    labelDot->setText("<b>.</b>");

    extemplate = new KMyHistoryCombo( page_4 );
    extemplate->setEnabled( FALSE );

    buttonHelp = new KPushButton( page_4 );
    buttonHelp->setPixmap( SmallIcon("help") );
    
    checkExtension = new QCheckBox( page_4 );
    checkExtension->setText( i18n( "&Use extension of the input file" ) );
    checkExtension->setChecked( TRUE );
    
    groupNumber = new QButtonGroup( page_4 );
    groupNumber->setTitle( i18n( "Numbers" ) );
    groupNumber->setColumnLayout(0, Qt::Vertical );
    groupNumber->layout()->setSpacing( 6 );
    groupNumber->layout()->setMargin( 11 );
    groupNumberLayout = new QVBoxLayout( groupNumber->layout() );
    groupNumberLayout->setAlignment( Qt::AlignTop );

    spinIndex = new KIntNumInput( groupNumber );
    spinIndex->setMaxValue( SPINMAX );
    spinIndex->setLabel( i18n( "Start index:" ) ); //, AlignVCenter );

    spinStep = new KIntNumInput( spinIndex, 1, groupNumber );
    spinStep->setRange( -10, 40000, 1, false );
    spinStep->setValue( 1 );
    spinStep->setLabel( i18n( "Step by:" ) );//, AlignVCenter );

    buttonSkip = new KPushButton( groupNumber );
    buttonSkip->setText( i18n( "Skip &Numbers" ) );

    buttonReplace = new KPushButton( i18n("&Find and replace"), page_4 );
    buttonReplace->setIconSet( SmallIconSet("find") );

    buttonCoord = new KPushButton( i18n("&Insert part of filename"), page_4 );

    helpDialog = new HelpDialog( filename->lineEdit() );
    helpDialog->addHeadline( i18n("Build in Functions:" ) );
    helpDialog->add("$", i18n("old filename") );
    helpDialog->add("%", i18n("old filename converted to lower case") );
    helpDialog->add("%", i18n("old filename converted to upper case") );
    helpDialog->add("*", i18n("first letter of every word upper case") );
    helpDialog->add("", i18n("first letter of filename upper case") );
    helpDialog->add("#", i18n("number (try also ##, ###, ... for leading zeros)") );
    helpDialog->add("/", i18n("create a sub directory") );
    helpDialog->add("\\", i18n("strip whitespaces") );
    helpDialog->add("[x-y]", i18n("character x to y of old filename") );
    helpDialog->add("[x;y]", i18n("y characters of old filename starting at x") );
    helpDialog->add("[dirname]", i18n("insert name of directory") );
    helpDialog->add("[dirname.]", i18n("insert name of directories top level directory") );
    helpDialog->addHeadline( i18n("Date and Time Functions:" ) );
    helpDialog->add("[date]", i18n("current date") );
    helpDialog->add("[year]", i18n("year") );
    helpDialog->add("[month]", i18n("month") );
    helpDialog->add("[day]", i18n("day") );
    helpDialog->add("[time]", i18n("current time") );
    helpDialog->add("[hour]", i18n("hour") );
    helpDialog->add("[minute]", i18n("minute") );
    helpDialog->add("[second]", i18n("second") );

    description5 = new QLabel( page_4 );
    description5->setText( i18n(
    "<b>$</b> old filename<br><b>%</b> old filename"
    " converted to lower case<br><b>&</b> old filename converted to upper case<br>"
    "<b>*</b> first letter of every word uppercase<br>" 
    "<b>#</b> Adds a number to the filename (add more #'s for leading zeros)<br>"
    "<b>/</b> creates a subdirectory<br>"
    "<b>[x-y]</b> Characters x to y of the old filename<br>"
    "<b>[%x-y]</b> Characters x to y converted to lower case" ) );
    description5->setAlignment( int( QLabel::WordBreak | QLabel::AlignTop ) );

    preview = new KMyListView( &changes, fileList, page_4 );
    preview->setSorting( -1 );
    preview->addColumn( i18n( "Origin" ) );
    preview->addColumn( i18n( "Renamed" ) );
    preview->setColumnWidthMode( 0, QListView::Manual );
    preview->setColumnWidthMode( 1, QListView::Manual );

    buttonUp2 = new KPushButton( page_4 );
    buttonUp2->setPixmap( BarIcon( "1uparrow" ) );

    buttonDown2 = new KPushButton( page_4 );
    buttonDown2->setPixmap( BarIcon( "1downarrow" ) );

    Layout13->addWidget( labelTemplate );
    Layout13->addWidget( filename );
    Layout13->setStretchFactor( filename, 2 );
    Layout13->addWidget( labelDot );
    Layout13->addWidget( extemplate );
    Layout13->addWidget( buttonHelp );
    Layout12->addLayout( Layout13 );

    QHBoxLayout* LayoutN = new QHBoxLayout( 0, 6, 6 );
    LayoutN->addWidget( spinIndex );
    LayoutN->addWidget( spinStep );
    groupNumberLayout->addLayout( LayoutN );
    groupNumberLayout->addWidget( buttonSkip );
    
    Layout18->addWidget( buttonReplace );
    Layout18->addWidget( buttonCoord );
    Layout18->addItem( spacer_9 );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
       
    Layout12->addWidget( checkExtension );
    Layout12->addWidget( groupNumber );
    Layout12->addItem( spacer );
    Layout12->addLayout( Layout18 );

    Layout17->addWidget( description5 );
    Layout17->addItem( spacer );

    Layout11->addLayout( Layout12 );
    Layout11->addItem( spacer );
    Layout11->addLayout( Layout17 );

    Layout23->addItem( spacer );
    Layout23->addWidget( buttonUp2 );
    Layout23->addWidget( buttonDown2 );
    Layout23->addItem( spacer );

    Layout22->addWidget( preview );
    Layout22->addLayout( Layout23 );
    Layout22->setStretchFactor( preview, 2 );

    Layout10->addLayout( Layout11 );
    Layout10->addLayout( Layout22 );

    pageLayout_4->addLayout( Layout10 );
    addPage( page_4, pageTitle[3] );

    QToolTip::add( filename, i18n( "Add a template.<br>Example: <i>picture###</i>" ) );
    QToolTip::add( extemplate, i18n( "Add a template for the file extension.<br>It behaves like the filename template.") );
    QToolTip::add( spinIndex, i18n( "Number of the first file.<br>The files are always numbered the same way as they are shown in the result list." ) );
    QToolTip::add( spinStep, i18n( "The counter is increased/decreased by this value." ) );
    QToolTip::add( buttonSkip, i18n( "Select numbers that should be skipped." ));
    QToolTip::add( checkExtension, i18n("Checking this checkbox the same as setting the extension template to $"));
    QToolTip::add( buttonUp2, i18n( "Move selected items up" ));
    QToolTip::add( buttonDown2, i18n( "Move selected items down" ));
    QToolTip::add( preview, i18n( "Double click on an item to modify it." ));
    QToolTip::add( buttonHelp, i18n("Help Dialog with all tokens supported by krename."));
}

void wizard::setupTab1()
{
    /*
     * Draw plugin user interfaces
     */

    QWidget* parent;
    QVBoxLayout* Layout;
    QVBoxLayout* gLayout;
    QGroupBox* g;

    // Memory leak ?
    checkPlugins.resize( plugin->libs.count() );

    QDictIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
    int z = 0;
    while ( it.current() ) {
        QString i = it.currentKey();
        if(!plugin->libs[i]->isActive && !checkShowAll->isChecked() ) {
            ++it;
            z++;
            continue;
        }

        parent = new QWidget( tab, "parent" );
        Layout = new QVBoxLayout( parent, 11, 6, "Layout");
        g = new QGroupBox( parent );
        gLayout = new QVBoxLayout( g, 11, 6, "gLayout" );
        checkPlugins[z] = new QCheckBox( i18n("&Use this plugin"), parent );

        connect( checkPlugins[z], SIGNAL( clicked() ), this, SLOT( pluginHelpChanged() ) );

        plugin->libs[i]->plugin->drawInterface( g, gLayout );

        Layout->addWidget( checkPlugins[z] );
        Layout->addWidget( g );

        tab->addTab( parent, plugin->libs[i]->plugin->getAccelName() );

        ++it;
        z++;
    }

    if( tab->count() == 0 ) {
        parent = new QWidget( tab );

        tab->addTab( parent, i18n("No Plugin Loaded") );
    }
}

void wizard::about()
{
    KAboutApplication * d = new KAboutApplication( kapp->aboutData(), this );
    d->show( this );
}

void wizard::addFile()
{
    QStringList slist( KFileDialog::getOpenFileNames( lastPath, "*", this ));
    if (!slist.isEmpty() ) {
        fileList->startAdding();
        QStringList::Iterator it = slist.begin();
        lastPath = QFileInfo((*it).latin1()).dirPath( true );
        for ( ; it != slist.end(); ++it )
            fileList->addFile( (*it).latin1() );
        fileList->endAdding();
        fileList->setPreview( checkPreview->isChecked() );
    }
    updateCount();
    updatePreview();
}

void wizard::addDir()
{
    QString s (KFileDialog::getExistingDirectory ( lastPath, this ));
    if(!s.isEmpty()) {
        fileList->startAdding();
        
        if( m_dir )
            fileList->addDirName( s );

        fileList->addDir( s, false );
        fileList->endAdding();
        
        lastPath = s;
        updatePreview();
    }
}

void wizard::addDirRec()
{
    KFileDialog* fd = new KFileDialog( lastPath, QString("*"), this, "fd", true );
    fd->setMode( KFile::Directory );
    fd->setOperationMode( KFileDialog::Opening );
    if( fd->exec() == QDialog::Accepted ) {
        QString s = lastPath = fd->selectedFile();
        if(!s.isEmpty()) {
            fileList->startAdding();
            fileList->addDir( s, true, true, fd->currentFilter(), m_dir );
            fileList->endAdding();
            updatePreview();
        }
    }
}

void wizard::addDirName()
{
    KFileDialog* fd = new KFileDialog( lastPath, QString::null, this, "fd", true );
    fd->setMode( KFile::Directory );
    fd->setOperationMode( KFileDialog::Opening );
    if( fd->exec() == QDialog::Accepted ) {
        lastPath = fd->selectedFile();
        QStringList s = fd->selectedFiles();
        fileList->startAdding();
        for( unsigned int i = 0; i < s.count(); i++ )
            fileList->addDirName( s[i] );
        fileList->endAdding();
        updatePreview();
    }
}

void wizard::clearList()
{
    fileList->clear();
    updateCount();
    updatePreview();
}

void wizard::removeFile()
{
    unsigned int i = 0;
    do {
        if(fileList->isSelected( i ))
            fileList->removeItem( i );
        else
            i++;
    } while( i < fileList->count() );

    updateCount();
    updatePreview();
}

void wizard::removeFile( int index )
{
    fileList->removeItem( index );
    updateCount();
    updatePreview();
}

void wizard::help()
{
    kapp->invokeHelp();
}

void wizard::start()
{
    if(!checkErrors())
        return;

    // Let the plugins check for erorrs

    QDictIterator<PluginLoader::PluginLibrary> it( plugin->libs );
    while ( it.current() ) {
        QString i = it.currentKey();
        if( plugin->libs[i]->isActive && plugin->libs[i]->usePlugin )
            if( !plugin->libs[i]->plugin->checkError() )
                return;
        ++it;
    }

    ProgressDialog* p = new ProgressDialog( 0, "p" );
    p->print(QString( i18n("Starting conversion of %1 files.") ).arg(fileList->count()));

    values* val = new values;
    fillStructures( val );

    /*
     * Structures are filled, save config now!
     */
    filename->addToHistory( filename->text() );
    dirname->addToHistory( dirname->text() );
    extemplate->addToHistory( extemplate->text() );

    // Save History
    KConfig * config = kapp->config();

    config->setGroup("Path");
    config->writeEntry( "History", dirname->historyItems() );
    config->writeEntry( "CompletionItems", dirname->completionObject()->items() );
    config->writeEntry( "CompletionMode", (int) dirname->completionMode() );

    config->setGroup("Template");
    config->writeEntry( "History", filename->historyItems() );
    config->writeEntry( "CompletionItems", filename->completionObject()->items() );
    config->writeEntry( "CompletionMode", (int) filename->completionMode() );

    config->setGroup("ExTemplate");
    config->writeEntry( "History", extemplate->historyItems() );
    config->writeEntry( "CompletionItems", extemplate->completionObject()->items() );
    config->writeEntry( "CompletionMode", (int) extemplate->completionMode() );

    config->sync();

    // save the configuration
    saveConfig();

    b = new BatchRenamer();

    p->print( "" );
    p->print( "========" );
    p->print( "" );
    p->show();

    b->processFiles( val, p, plugin );
    hide();
    // Memory leak!!! :
//    delete this;
}

void wizard::showPage( QWidget* page )
{
    QWizard::showPage( page );

    if ( page == page_4 ) {
        finishButton()->setEnabled( true );
        finishButton()->setDefault( true );
    }

    if( topLabel1.size() ) {
        disableControls();
        for( unsigned int i = 0; i < topLabel1.size(); i++ )
            if( page == page_1 ) {
                topLabel1[i]->setCurrent( true );
            } else if( page == page_2 ) {
                topLabel2[i]->setCurrent( true );
            } else if( page == page_3 ) {
                topLabel3[i]->setCurrent( true );
            } else if( page == page_4 ) {
                topLabel4[i]->setCurrent( true );
            }
    }
}

void wizard::accept()
{ /* Do nothing */ }

void wizard::reject()
{
    this->close();
}

void wizard::closeEvent( QCloseEvent* e )
{
    if( m_ask )
        if( KMessageBox::questionYesNo( this, i18n("Do you really want to quit Krename ?") ) == KMessageBox::Yes ) {
            e->accept();
            KApplication::exit( 0 );
        } else
            e->ignore();
    else
        e->accept();
}

void wizard::enableControls()
{
    urlrequester->setEnabled( !optionRename->isChecked() );
    undorequester->setEnabled( checkUndoScript->isChecked() );
    extemplate->setEnabled( !checkExtension->isChecked() );
}

bool wizard::checkErrors()
{
    if( filename->text().isEmpty() ) {
        KMessageBox::sorry( this, i18n("Specify a template to use for renaming files.") );
        return false;
    }
    if( dirname->text().isEmpty() && !optionRename->isChecked()) {
        KMessageBox::sorry( this, i18n("Please give a destination directory !") );
        return false;
    }
    if( fileList->count() == 0 ) {
        KMessageBox::sorry( this, i18n("Please add some files !") );
        return false;
    }
    if( !optionRename->isChecked() ) {
        if( dirname->text().right(1).compare("/") )
            dirname->setText(QString("%1/").arg(dirname->text()));

        KFileItem fileItem( KFileItem::Unknown, KFileItem::Unknown, dirname->text());
        if (!fileItem.isDir()) {
            KMessageBox::sorry( this, i18n("The given destination directory does not exist!") );
            return false;
        }
    }
    if( checkUndoScript->isChecked() && undorequester->url().isEmpty() ) {
        KMessageBox::sorry( this, i18n("Please give the name of the undo script!") );
        return false;
    }

    return true;
}

void wizard::updateCount()
{
    labelCount->setText( QString( i18n("Files: <b>%1</b>") ).arg(fileList->count()));
}

void wizard::fillStructures( values* v )
{
    QFileInfo fi;
    for( unsigned int i = 0; i < fileList->count(); i++) {
        fi.setFile( fileList->text( i ) );

        data fdata;
        if( !fi.isDir() ) {
            splitFilename( &fi, &fdata.source_name, &fdata.extension );
        } else {
            //fi.isDir() == true//
            fdata.source_name = BatchRenamer::findDirName( "dirname", fileList->text( i ) );
            fdata.extension = "";
        }

        /* Let's do some error checking for files
         * called: file_xy.
         */
        if( fileList->text( i ).right(1) == "." )
            // File ends wizth a dot
            fdata.source_name.append( "." );

        if( !fdata.extension.isEmpty() && fdata.extension.left(1) != "." )
            fdata.extension.prepend( "." );

        fdata.source_dir = fi.dirPath( true );
        if( fdata.source_dir.right(1) != "/" )
            fdata.source_dir.append("/");

        v->files.append( fdata );
    }

    v->text = filename->text();
    v->dirname = dirname->text();
    if( checkExtension->isChecked() )
        v->extext = "$";
    else
        v->extext = extemplate->text();

    v->index = spinIndex->value();
    v->step = spinStep->value();
    v->overwrite = checkOverwrite->isChecked();

    if( (v->undo = checkUndoScript->isChecked()) )
        v->undoScript = undorequester->url();

    v->skip = skip;
    v->replace = rep;
    v->changes = changes;

    if( optionCopy->isChecked() )
        v->m = COPY;
    else if( optionMove->isChecked() )
        v->m = MOVE;
    else if( optionRename->isChecked() )
        v->m = RENAME;
    else if( optionLink->isChecked() )
        v->m = LINK;

    /*
     * Tell plugins that they should fill their structure, too.
     * Check also which plugins should be used !
     */
    QDictIterator<PluginLoader::PluginLibrary> it( plugin->libs );
    for( int z = 0; it.current(); ++it, z++ ) {
        QString i = it.currentKey();
        if( plugin->libs[i]->isActive || checkShowAll->isChecked() ) {
            plugin->libs[i]->usePlugin = checkPlugins[z]->isChecked();
            if( plugin->libs[i]->usePlugin )
                plugin->libs[i]->plugin->fillStructure();
        }
    }
}

void wizard::updatePre()
{
    updatePreview();
}

void wizard::updatePreview()
{
    if( noRealTimePreview )
        return;

    if( filename->isEmpty() && extemplate->isEmpty() && !checkExtension->isChecked() )
        return;

    if( fileList->count() <= 0 ) {
        preview->clear();
        return;
    }

    int h = preview->horizontalScrollBar()->value();
    int v = preview->verticalScrollBar()->value();

    BatchRenamer* b = new BatchRenamer;

    values* val = new values;

    fillStructures( val );
    preview->clear();

    b->createPreview( preview, val, plugin );

    preview->horizontalScrollBar()->setValue( h );
    preview->verticalScrollBar()->setValue( v );

    delete b;
}

void wizard::addFile( KURL filename )
{
    fileList->startAdding();
    fileList->addFile( filename.path() );
    fileList->endAdding();
    updateCount();
}

void wizard::addFile( QString filename)
{
    fileList->startAdding();
    fileList->addFile( filename );
    fileList->endAdding();
    updateCount();
}

void wizard::loadConfig()
{
    KConfig* config = kapp->config();

    config->setGroup("Path");
    QStringList histList = config->readListEntry("History");
    dirname->setHistoryItems( histList );

    QStringList compList = config->readListEntry("CompletionItems");
    if( compList.isEmpty() )
        dirname->completionObject()->setItems( histList );
    else
        dirname->completionObject()->setItems( compList );

    int mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
    dirname->setCompletionMode( (KGlobalSettings::Completion) mode );

    histList.clear();
    compList.clear();
    config->setGroup("Template");
    histList = config->readListEntry("History");
    filename->setHistoryItems( histList );

    compList = config->readListEntry("CompletionItems");
    if( compList.isEmpty() )
        filename->completionObject()->setItems( histList );
    else
        filename->completionObject()->setItems( compList );

    mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
    filename->setCompletionMode( (KGlobalSettings::Completion) mode );

    histList.clear();
    compList.clear();
    config->setGroup("ExTemplate");
    histList = config->readListEntry("History");
    extemplate->setHistoryItems( histList );

    compList = config->readListEntry("CompletionItems");
    if( compList.isEmpty() )
        extemplate->completionObject()->setItems( histList );
    else
        extemplate->completionObject()->setItems( compList );

    mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
    extemplate->setCompletionMode( (KGlobalSettings::Completion) mode );

    config->setGroup("GUISettings");
    // Image Preview
    checkPreview->setChecked(config->readBoolEntry("ImagePreview", false ));
    fileList->setPreview( checkPreview->isChecked() );
    spinIndex->setValue(config->readNumEntry("StartIndex", 0));
    m_ask = config->readBoolEntry("AskOnExit", true );
    m_dir = config->readBoolEntry("AddDirNames", false );
    fileList->setHiddenDirs( config->readBoolEntry("AddHiddenDirs", false ));
    m_loadplugins = config->readBoolEntry("LoadFilePlugins", true );
    checkShowAll->setChecked(config->readBoolEntry("ShowAllPlugins", true));
    checkName->setChecked(config->readBoolEntry("Name", false ));
    fileList->setPreviewSize( config->readNumEntry("PreviewSize", 80) );
    fileList->setName( checkName->isChecked() );
    CoordDialog::m_inversion = config->readBoolEntry("Inverse", false);
    preview->setColumnWidth( 0, config->readNumEntry("Column0", preview->width()/2 - 2 ) );
    preview->setColumnWidth( 1, config->readNumEntry("Column1", preview->width()/2 - 2 ) );
    
    if( !checkPreview->isChecked() ) {
        checkName->setChecked( true );
        checkName->setEnabled( false );
    }
    
    if( config->readBoolEntry("Maximized", false ) )
        showMaximized();
    else
        resize( config->readNumEntry( "Width", width() ), config->readNumEntry( "Height", height() ) );
    

    // Visible Plugins
    QStringList tmp = config->readListEntry( "VisiblePlugins" );
    QString t;

    QDictIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
    if( !tmp.isEmpty() )
        for ( QStringList::Iterator sit = tmp.begin(); sit != tmp.end(); ++sit ) {
            it.toFirst();
            while ( it.current() ) {
                QString i = it.currentKey();
                if( (*sit).latin1() == plugin->libs[i]->plugin->getName() )
                    plugin->libs[i]->isActive = true;
                ++it;
            }
        }

    it.toFirst();
    tmp.clear();

    // Invisible Plugins
    tmp = config->readListEntry( "InVisiblePlugins" );
    if( !tmp.isEmpty() )
        for ( QStringList::Iterator sit = tmp.begin(); sit != tmp.end(); ++sit ) {
            it.toFirst();
            while ( it.current() ) {
                QString i = it.currentKey();
                if( (*sit).latin1() == plugin->libs[i]->plugin->getName() )
                    plugin->libs[i]->isActive = false;
                ++it;
            }
        }

    // Skip
    skip = config->readIntListEntry( "skip" );

    // Replace Strings
    QStringList find, replace;
    QValueList<int> regular;
    find = config->readListEntry( "FindList" );
    replace = config->readListEntry( "ReplaceList" );
    regular = config->readIntListEntry( "RegularList" );
    if( !find.isEmpty() && !replace.isEmpty() )
        for( unsigned int i = 0; i < find.count(); i++ ) {
            replacestrings rs;
            rs.find = find[i];
            rs.replace = replace[i];
            rs.reg = regular[i];
            rep.append( rs );
        }

    // File Extension
    config->setGroup("FileExtension");
    fileExtensionMatrix[0] = config->readBoolEntry( "FirstDot", true );
    fileExtensionMatrix[1] = config->readBoolEntry( "LastDot", false );
    fileExtensionMatrix[2] = config->readBoolEntry( "CustomDot", false );
    fileExtensionInt = config->readNumEntry( "CustomDotValue", 0 );

    // visible pages
    config->setGroup("WizardPages");
    mnuView->setItemChecked( ID_PAGE_1, config->readBoolEntry("Page1", true ) );
    mnuView->setItemChecked( ID_PAGE_2, config->readBoolEntry("Page2", true ) );
    mnuView->setItemChecked( ID_PAGE_3, config->readBoolEntry("Page3", true ) );
    pageVisible1();
    pageVisible2();
    pageVisible3();
    m_title = config->readBoolEntry("AdvancedTitle", true );
    
    updatePluginTab();
}

void wizard::saveConfig()
{
    KConfig* config = kapp->config();

    config->setGroup("GUISettings");
    config->writeEntry("ImagePreview", checkPreview->isChecked() );
    config->writeEntry("StartIndex", spinIndex->value() );
    config->writeEntry("ShowAllPlugins", checkShowAll->isChecked() );
    config->writeEntry("Maximized", isMaximized() );
    config->writeEntry("Width", width() );
    config->writeEntry("Height", height() );
    config->writeEntry("AskOnExit", m_ask );
    config->writeEntry("AddDirNames", m_dir );
    config->writeEntry("AddHiddenDirs", fileList->hiddenDirs() );
    config->writeEntry("LoadFilePlugins", m_loadplugins );
    config->writeEntry("Name", checkName->isChecked() );
    config->writeEntry("PreviewSize", fileList->getPreviewSize() );
    config->writeEntry("Inverse", CoordDialog::m_inversion );
    config->writeEntry("Column0", preview->columnWidth( 0 ) );
    config->writeEntry("Column1", preview->columnWidth( 1 ) );
    
    // Save visible plugins:
    QStringList tmp;
    QDictIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
    for( ; it.current(); ++it ) {
        QString i = it.currentKey();
        if( plugin->libs[i]->isActive )
            tmp.append( plugin->libs[i]->plugin->getName() );
    }
    config->writeEntry("VisiblePlugins", tmp );

    // Save invisible plugins:
    for( it.toFirst(), tmp.clear(); it.current(); ++it ) {
        QString i = it.currentKey();
        if( !plugin->libs[i]->isActive )
            tmp.append( plugin->libs[i]->plugin->getName() );
    }
    config->writeEntry("InVisiblePlugins", tmp );

    config->setGroup("FileExtension");
    config->writeEntry( "FirstDot", fileExtensionMatrix[0] );
    config->writeEntry( "LastDot", fileExtensionMatrix[1] );
    config->writeEntry( "CustomDot", fileExtensionMatrix[2] );
    config->writeEntry( "CustomDotValue", fileExtensionInt );

    // visible pages
    config->setGroup("WizardPages");
    config->writeEntry( "Page1", mnuView->isItemChecked( ID_PAGE_1 ) );
    config->writeEntry( "Page2", mnuView->isItemChecked( ID_PAGE_2 ) );
    config->writeEntry( "Page3", mnuView->isItemChecked( ID_PAGE_3 ) );
    config->writeEntry( "AdvancedTitle", m_title );
                
    config->sync();
}

void wizard::addDir( QString dir )
{
    fileList->startAdding();
    fileList->addDir( dir, true );
    fileList->endAdding();
}

void wizard::addDirName( QString dirname )
{
    fileList->startAdding();
    fileList->addDirName( dirname );
    fileList->endAdding();
}

void wizard::skipDialog()
{
    SkipDialog* s = new SkipDialog( skip, spinIndex->value(), this, "s", true );
    if( s->exec() == QDialog::Accepted )
        skip = s->getList();

    updatePreview();
}

void wizard::replace()
{
    ReplaceDialog* r = new ReplaceDialog( rep, this, "r", true );
    if( r->exec() == QDialog::Accepted )
        rep = r->getList();

    updatePreview();
}

void wizard::toggleImagePreview()
{
    KApplication::setOverrideCursor( Qt::waitCursor );
    fileList->setPreview( checkPreview->isChecked() );
    checkName->setEnabled( checkPreview->isChecked() );
    if( !checkPreview->isChecked() )
        checkName->setChecked( true );
    fileList->setName( checkName->isChecked() );
    KApplication::restoreOverrideCursor();
}

void wizard::toggleName()
{
    KApplication::setOverrideCursor( Qt::waitCursor );
    fileList->setName( checkName->isChecked() );
    KApplication::restoreOverrideCursor();
}

void wizard::preferences()
{
    ConfDialog* cd = new ConfDialog( plugin, this, "cd", true );
    cd->setTitle( m_title );
    cd->setLoadPlugins( m_loadplugins );
    cd->setAsk( m_ask );
    cd->setDir( m_dir );
    cd->setHiddenDir( fileList->hiddenDirs() );
    cd->setThumbSize( fileList->getPreviewSize() );

    cd->optionFirstDot->setChecked( fileExtensionMatrix[0] );
    cd->optionLastDot->setChecked( fileExtensionMatrix[1] );
    cd->optionCustomDot->setChecked( fileExtensionMatrix[2] );
    cd->spinDot->setValue( fileExtensionInt );
    
    if( cd->exec() == QDialog::Accepted ) {
        m_title = cd->title();
        m_loadplugins = cd->loadplugins();
        m_ask = cd->ask();
        m_dir = cd->dir();
        fileList->setHiddenDirs( cd->hiddendir() );

        if( fileList->getPreviewSize() != cd->thumbSize() ) {
            fileList->setPreviewSize( cd->thumbSize() );
            fileList->setPreview( checkPreview->isChecked() );
        }

        fileExtensionMatrix[0] = cd->optionFirstDot->isChecked();
        fileExtensionMatrix[1] = cd->optionLastDot->isChecked();
        fileExtensionMatrix[2] = cd->optionCustomDot->isChecked();
        fileExtensionInt = cd->spinDot->value();

        updatePreview();
        updatePluginTab();        
    }

//    delete cd;
}

void wizard::updatePluginTab()
{
    do {
        tab->removePage( tab->page( 0 ) );
    } while( tab->count() > 0 );

    setupTab1();
    pluginHelpChanged();
    tab->setCurrentPage( 0 );
}

void wizard::moveUp()
{
    QListViewItem* item = preview->selectedItem();
    if(item) {
        int i = preview->itemIndex( item );
        fileList->moveUp( i );
        updatePreview();
        if( (i-1) > 0 ) {
            preview->setCurrentItem( preview->itemAtIndex( i-1 ) );
            preview->ensureItemVisible( preview->itemAtIndex( i-1 ) );
        }
    }
}

void wizard::moveDown()
{
    QListViewItem* item = preview->selectedItem();
    if(item) {
        int i = preview->itemIndex( item );
        fileList->moveDown( i );
        updatePreview();
        preview->setCurrentItem( preview->itemAtIndex( i+1 ) );
        preview->ensureItemVisible( preview->itemAtIndex( i+1 ) );
    }
}

void wizard::undo()
{
    UndoDialog* u = new UndoDialog( this );
    u->exec();
}

void wizard::changeUndoScriptName()
{
    const char* EXTENSION = ".krename";
    QString script = undorequester->url();
    if( script.right( 8 ) != EXTENSION ) {
        script += EXTENSION;
        undorequester->setURL( script );
    }
}

void wizard::pluginHelpChanged()
{
    QDictIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
    int z = 0;
    while ( it.current() ) {
        QString i = it.currentKey();
        if(!plugin->libs[i]->isActive && !checkShowAll->isChecked()) {
            ++it;
            z++;
            continue;
        }

        plugin->libs[i]->plugin->removeHelp( helpDialog );
        if( checkPlugins[z]->isChecked() ) 
            plugin->libs[i]->plugin->addHelp( helpDialog );

        
        ++it;
        z++;
    }

}

void wizard::layOutTitleRow( QHBoxLayout * layout, const QString & title )
{
    /* do nothing! */
}

void wizard::addTitle( QWidget* parent, QVBoxLayout* layout, QString title )
{
    QHBoxLayout * l = new QHBoxLayout( 6 );
    QSpacerItem* spacer = new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum );

    if( m_title ) {
        int index = pageCount();
        topLabel1[index] = new MyLabel( pageTitle[0], parent );
        topLabel2[index] = new MyLabel( pageTitle[1], parent );
        topLabel3[index] = new MyLabel( pageTitle[2], parent );
        topLabel4[index] = new MyLabel( pageTitle[3], parent );

        connect( topLabel1[index], SIGNAL(clicked()), this, SLOT(showPage1()));
        connect( topLabel2[index], SIGNAL(clicked()), this, SLOT(showPage2()));
        connect( topLabel3[index], SIGNAL(clicked()), this, SLOT(showPage3()));
        connect( topLabel4[index], SIGNAL(clicked()), this, SLOT(showPage4()));

        l->addWidget( topLabel1[index] );
        l->addWidget( new QLabel( "->", parent ) );
        l->addWidget( topLabel2[index] );
        l->addWidget( new QLabel( "->", parent ) );
        l->addWidget( topLabel3[index] );
        l->addWidget( new QLabel( "->", parent ) );
        l->addWidget( topLabel4[index] );
        l->addItem( spacer );
    } else
        l->addWidget( new QLabel( title, parent ), 10 );

    QFrame* hbar1 = new QFrame( parent, "<hr>", 0 );
    hbar1->setFrameStyle( QFrame::Sunken + QFrame::HLine );
    hbar1->setFixedHeight( 12 );

    layout->addLayout( l, 0 );
    layout->addWidget( hbar1 );
}

void wizard::visiblePage( int id )
{
     mnuView->setItemChecked( id, !mnuView->isItemChecked( id ) );
}

void wizard::showPage1()
{
    showPage( page_1 );
}

void wizard::showPage2()
{
    showPage( page_2 );
}

void wizard::showPage3()
{
    showPage( page_3 );
}

void wizard::showPage4()
{
    showPage( page_4 );
}

void wizard::switchTitle( bool b )
{
    KMessageBox::information( this,
        i18n("This change will take effect when you restart KRename the next time."), 0,
        "AdvancedTitleInfo" );
}

void wizard::disableControls()
{
    if( !topLabel1.size() )
        return;

    for( unsigned int i = 0; i < PAGES; i++ ) {                
        topLabel1[i]->setCurrent( false );
        topLabel2[i]->setCurrent( false );
        topLabel3[i]->setCurrent( false );
        topLabel4[i]->setCurrent( false );
    }
}

void wizard::showTokenHelp()
{
    helpDialog->show();
}

void wizard::pageVisible1()
{
    setAppropriate( page_1, mnuView->isItemChecked( ID_PAGE_1 ) );
}

void wizard::pageVisible2()
{
    setAppropriate( page_2, mnuView->isItemChecked( ID_PAGE_2 ) );
}

void wizard::pageVisible3()
{
    setAppropriate( page_3, mnuView->isItemChecked( ID_PAGE_3 ) );
}

void wizard::spellCheck()
{
//    KSpellConfig cfg;
//    KSpell* spell = new KSpell( 0L, i18n("Spell Checker"), 0 , 0, 0, true, true );
//
//    for( int i = 0; i < preview->childCount(); i++ ) {
//        QString text = preview->itemAtIndex( i )->text( 0 );
//        QString tmp = text;
//
//        spell->check( text );
////        while (spell->status()!=KSpell::Finished)
////          kapp->processEvents();
//
//        if( text != tmp ) {
//            manualchanges tmp;
//            tmp.filename = fileList->text( i );
//            tmp.user = text;
//            changes.append( tmp );
//        }
//    }
//    delete spell;
//
//    updatePreview();
}

void wizard::getCoordinates()
{
    QString name;
    QListViewItem* item = preview->selectedItem();
    if( item )
        name = item->text( 0 );
    else {
        item = preview->itemAtIndex( 0 );
        if( item )
            name = item->text( 0 );
    }

    QFileInfo fi( name );
    QString extension;
    splitFilename( &fi, &name, &extension );
    CoordDialog* cd = new CoordDialog( name, this );
    if( cd->exec() == QDialog::Accepted ) {
        QString t = filename->text();

        t.insert( filename->cursorPosition(), cd->coords() );
        filename->setText( t );    
    }

    delete cd;
}

void wizard::splitFilename( QFileInfo* fi, QString* base, QString* extension )
{
    if( fileExtensionMatrix[0] ) {
        *base = fi->baseName();
        *extension = fi->extension( true );
    } else if( fileExtensionMatrix[1] ) {
        *extension = fi->extension( false );
        if( !extension->isEmpty() )
            *base = fi->fileName().mid( 0, fi->fileName().length() - extension->length() - 1 );
        else
            *base = fi->baseName();
    } else if( fileExtensionMatrix[2] ) {
        *base = FileOperation::getBaseName( fi->filePath(), fileExtensionInt );
        *extension = FileOperation::getExtension( fi->filePath(), fileExtensionInt );
    }
}

