/***************************************************************************
                          wizard.h  -  description
                             -------------------
    begin                : Die Mai 15 15:34:19 CEST 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef WIZARD_H
#define WIZARD_H

// KDE includes
#include <kwizard.h>

#include <vector>

// Own includes
#include "batchrenamer.h"

class HelpDialog;
class KIntNumInput;
class KMyHistoryCombo;
class KMyListBox;
class KMyListView;
class KMenuBar;
class KPopupMenu;
class KPushButton;
class KToggleAction;
class KURL;
class KURLRequester;
class MyHelpCombo;
class MyLabel;
class PluginLoader;
class QButtonGroup;
class QCheckBox;
class QGroupBox;
class QFileInfo;
class QFrame;
class QHBoxLayout;
class QLabel;
class QListView;
class QRadioButton;
class QTabWidget;
class QVBoxLayout;
class QWidget;
class wizard : public KWizard
{ 
    Q_OBJECT

    public:
        wizard( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
        ~wizard();

        void addFile( KURL filename );
        void addFile( QString filename);
        void addDir( QString dir );
        void addDirName( QString dirname );
        void showPage(QWidget* page);

        /*
         * public because it gets called from ProgressDialog::again()
         */
        void updatePre();

        static bool noRealTimePreview;

    private slots:
        void about();
        void accept();
        void addDir();
        void addDirRec();
        void addDirName();
        void addFile();
        bool checkErrors();
        void clearList();
        void enableControls();
        void toggleImagePreview();
        void moveUp();
        void moveDown();
        void help();
        void reject();
        void removeFile();
        void removeFile( int index );
        void preferences();
        void replace();
        void saveConfig();
        void skipDialog();
        void start();
        void switchTitle(bool b);
        void updateCount();
        void updatePreview();
        void undo();
        void changeUndoScriptName();
        void pluginHelpChanged();
        void updatePluginTab();
        void getCoordinates();

        void visiblePage(int id);

        void showPage1();
        void showPage2();
        void showPage3();
        void showPage4();

        void pageVisible1();
        void pageVisible2();
        void pageVisible3();

        void showTokenHelp();
        void spellCheck();
        void toggleName();

    private:
        /*
         * Save the last path of a KFileDialog
         */
        static QString lastPath;

        void loadConfig();
        void addTitle( QWidget* parent, QVBoxLayout* layout, QString title );
        void fillStructures( values* v );
        void splitFilename( QFileInfo* fi, QString* base, QString* extension );
        
        void setupActions();
        void setupPage1();
        void setupPage2();
        void setupPage3();
        void setupPage4();
        void setupTab1();
        void setupPages();

        QValueList<manualchanges> changes;
        bool fileExtensionMatrix[2];
        int fileExtensionInt;
        void disableControls();

    protected:
        void layOutTitleRow( QHBoxLayout * layout, const QString & title );
        void closeEvent( QCloseEvent* e );

        PluginLoader* plugin;

        QTabWidget* tab;
        QTabWidget* pluginTab;
        QWidget* page_1;
        QWidget* page_2;
        QWidget* page_3;
        QWidget* page_4;
        QWidget* tab0;
        QWidget* tab1;
        QWidget* tab2;
        QWidget* tab3;

        KMenuBar* menuBar;
        KPopupMenu* mnuView;

        bool m_loadplugins;
        bool m_title;
        bool m_ask;
        bool m_dir;
                
        KPushButton* buttonUp;
        KPushButton* buttonDown;
        KPushButton* buttonUp2;
        KPushButton* buttonDown2;
        KPushButton* buttonAdd;
        KPushButton* buttonAddDir;
        KPushButton* buttonAddDirRec;
        KPushButton* buttonAddDirName;
        KPushButton* buttonRemove;
        KPushButton* buttonRemoveAll;
        KPushButton* buttonAbout;
        KPushButton* buttonSkip;
        KPushButton* buttonReplace;
        KPushButton* buttonConf;
        KPushButton* buttonHelp;
        KPushButton* buttonMove;
        KPushButton* buttonCoord;

        QLabel* description;
        QLabel* description2;
        QLabel* description3;
        QLabel* description4;
        QLabel* description5;

        QLabel* labelTemplate;
        QLabel* labelHelp;
        QLabel* labelCount;
        QLabel* labelPoint;

        std::vector <MyLabel*> topLabel1;
        std::vector <MyLabel*> topLabel2;
        std::vector <MyLabel*> topLabel3;
        std::vector <MyLabel*> topLabel4;

        KMyListBox* fileList;
        KMyListView* preview;
        QButtonGroup* groupOptions;

        QRadioButton* optionCopy;
        QRadioButton* optionMove;
        QRadioButton* optionLink;
        QRadioButton* optionRename;

        QGroupBox* groupDir;
        QGroupBox* groupExtension;
        QGroupBox* groupNumber;
        QGroupBox* groupUndo;

        KMyHistoryCombo* dirname;
        KURLRequester* urlrequester;
        KURLRequester* undorequester;
        KMyHistoryCombo* filename;
        KMyHistoryCombo* extemplate;

        KIntNumInput* spinIndex;
        KIntNumInput* spinStep;

        QCheckBox* checkName;
        QCheckBox* checkExtension;
        QCheckBox* checkOverwrite;
        QCheckBox* checkPreview;
        QCheckBox* checkUndoScript;
        QCheckBox* checkShowAll;
        std::vector<QCheckBox*> checkPlugins;

        MyHelpCombo* comboHelp;
        HelpDialog* helpDialog;
        
        BatchRenamer* b;
        QValueList<int> skip;
        QValueList<replacestrings> rep;

        // ===========
        // Layout:
        // ===========

        QVBoxLayout* pageLayout;
        QVBoxLayout* pageLayout_2;
        QVBoxLayout* pageLayout_3;
        QVBoxLayout* pageLayout_4;

        QHBoxLayout* tabLayout_0;
        QHBoxLayout* tabLayout_1;
        QHBoxLayout* tabLayout_2;
        QVBoxLayout* tabLayout_3;
        QVBoxLayout* groupAdvancedExtensionLayout;
        QHBoxLayout* groupOptionsLayout;
        QHBoxLayout* groupDirLayout;
        QVBoxLayout* groupUndoLayout;
        QVBoxLayout* groupNumberLayout;
        QHBoxLayout* groupExtensionLayout;

        // page1
        QVBoxLayout* Layout1;
        QHBoxLayout* Layout2;
        QVBoxLayout* Layout3;
        QHBoxLayout* Layout4;
        QVBoxLayout* Layout5;

        // page2
        QHBoxLayout* Layout7;
        QVBoxLayout* Layout8;

        // page4
        QVBoxLayout* Layout10;
        QHBoxLayout* Layout11;
        QVBoxLayout* Layout12;
        QHBoxLayout* Layout13;
        QHBoxLayout* Layout15;
        QVBoxLayout* Layout16;
        QVBoxLayout* Layout17;
        QHBoxLayout* Layout18;
        QVBoxLayout* Layout21;
        QHBoxLayout* Layout22;
        QVBoxLayout* Layout23;

        // tab
        QHBoxLayout* Layout100;
        QHBoxLayout* Layout101;
};

#endif // WIZARD_H
